%--------------------------------------------------------------------------
% Signals and Systems II - Exercise 1
%
% Sound example for the "encoded" speech signal.
%
%
% Nov. 2012, Nov. 2013, Nov. 2014
% Digital Signal Processing and System Theory
% Faculty of Engineering Christian-Albrechts-Universitt zu Kiel
% Jochen Withopf, jow@tf.uni-kiel.de
%--------------------------------------------------------------------------

clear

%--------------------------------------------------------------------------
% Set parameters, initialize variables
%--------------------------------------------------------------------------
fs = 100000; % has to be high enough for the modulated signals

% --- Minimum and maximum signal frequency
fmin =  300;
fmax = 3400;

% --- Modulation frequencies
f1    = 20000;
f2    = 25000;

% --- Filter cutoff frequencies (3dB cutoffs, we don't use stop band and
%     pass band edge frequencies for simplicity here) and filter order N
fc_hp = 20000;
fc_tp = 10000;
N     = 12;

% --- Design butterworth/Chebychev filters
% [b_hp a_hp] = butter( N, fc_hp/fs*2, 'high');
% [b_tp a_tp] = butter( N, fc_tp/fs*2, 'low');
[b_hp, a_hp] = cheby2( N, 30, fc_hp/fs*2, 'high');
[b_tp, a_tp] = cheby2( N, 30, fc_tp/fs*2, 'low');

% --- Load input signal, bandlimit and upsample it to sample rate fs;
%[u, fs_sig] = wavread( 'signals/interview_heino.wav' );
[u, fs_sig] = wavread( 'signals/sprachnotiz_1.wav' );

[b, a] = butter( 4, [fmin fmax]/fs_sig*2 );
u = filter( b, a, u );
u = resample( u, fs, fs_sig );

%--------------------------------------------------------------------------
% Modulation system
%--------------------------------------------------------------------------
t = linspace(0, length(u)/fs, length(u));
t = t(:);    % make t a column vector

g1 = u.*sin(2*pi*f1*t);
g2 = filter(b_hp, a_hp, g1);
g3 = g2.*sin(2*pi*f2*t);
g4 = filter(b_tp, a_tp, g3);

%--------------------------------------------------------------------------
% Plots and sound
%--------------------------------------------------------------------------

N_win = 1024*4;

[pu, f] = pwelch(u, N_win,128,N_win,fs);
pg1     = pwelch(g1,N_win,128,N_win,fs);
pg2     = pwelch(g2,N_win,128,N_win,fs);
pg3     = pwelch(g3,N_win,128,N_win,fs);
pg4     = pwelch(g4,N_win,128,N_win,fs);

% [pu, f] = pwelch(u,N_win,128,N_win,fs,'twosided');
% pg1    = pwelch(g1,N_win,128,N_win,fs,'twosided');
% pg2    = pwelch(g2,N_win,128,N_win,fs,'twosided');
% pg3    = pwelch(g3,N_win,128,N_win,fs,'twosided');
% pg4    = pwelch(g4,N_win,128,N_win,fs,'twosided');

figure(1);
plot(f/1000, 10*log10([pu pg1 pg2 pg3 pg4]), 'LineWidth', 1);
legend('|U(f)|', '|G_1(f)|', '|G_2(f)|', '|G_3(f)|', '|G_4(f)|');
grid on;
xlabel('Frequency in kHz');
ylabel('Magnitude in dB');

%--------------------------------------------------------------------------
% Spectrograms
%--------------------------------------------------------------------------
u_dn  = resample(u, fs_sig, fs);
g4_dn = resample(g4, fs_sig, fs);
M     = round(40/1000*fs_sig);
ovl   = round(M*0.9);
t_dn  = linspace(0, length(u_dn)/fs_sig, length(u_dn));

figure(2);
sp(1) = subplot(4,2,1);
plot(t_dn, u_dn);
title('Eingangssignal u(t)');
grid on;
xlim([t_dn(1) t_dn(end)]);

sp(2) = subplot(4,2,2);
plot(t_dn, g4_dn);
title('Ausgangssignal g_4(t)');
grid on;

sp(3) = subplot(4,2,[3,5,7]);
[S,F,T] = spectrogram(u_dn, M, ovl, M, fs_sig);
imagesc(T,F,20*log10(abs(S)));
axis xy
ylim([0 fs_sig/4]);
xlabel('Zeit in Sekunden');
ylabel('Frequenz in Hz');

sp(4) = subplot(4,2,[4, 6, 8]);
[S,F,T] = spectrogram(g4_dn, M, ovl, M, fs_sig);
imagesc(T,F,20*log10(abs(S)));
axis xy
ylim([0 fs_sig/4]);
xlabel('Zeit in Sekunden');
ylabel('Frequenz in Hz');

linkaxes(sp, 'x');

