%--------------------------------------------------------------------------
% Signals and Systems II - "Demo" for Problem 13
%
% Filter zero-mean white noise (generated by randn) with a narrow bandpass
% filter and investigate time signals, spectra and autocorrelations.
%
% Dec. 2010
% Revised: Jan. 2012, Jan. 2013
% Digital Signal Processing and System Theory
% Faculty of Engineering Christian-Albrechts-Universitt zu Kiel
% Jochen Withopf, jow@tf.uni-kiel.de
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
% Parameters
%--------------------------------------------------------------------------
N       = 4000;    % number of data points
P       = 8;       % half bandpass filter order
fc_low  = 0.2;     % lower cutoff frequency (normalized, between 0 and 1)
fc_high = 0.3;     % upper cutoff frequency (normalized, between 0 and 1)
maxlags = 250;     % number of lags to compute for autocorrelation

v = randn(N, 1);

%--------------------------------------------------------------------------
% Design highpass and lowpass filters and combine to a bandpass filter
%--------------------------------------------------------------------------
% --- Butterworth filter design
% [b_hp a_hp] = butter(P, fc_low, 'high');
% [b_lp a_lp] = butter(P, fc_high, 'low');

% --- Chebychev (type 1) filter design
[b_hp, a_hp] = cheby1(P, 2, fc_low, 'high');
[b_lp, a_lp] = cheby1(P, 2, fc_high, 'low');

% --- Combine high- and lowpass to bandpass
a_bp = conv(a_hp, a_lp);
b_bp = conv(b_hp, b_lp);

%--------------------------------------------------------------------------
% Apply bandpass filter
%--------------------------------------------------------------------------
y = filter(b_bp, a_bp, v);

%--------------------------------------------------------------------------
% Compute spectra and autocorrelations
%--------------------------------------------------------------------------
V_spec = fft(v);
Y_spec = fft(y);

[corr_v, lags_v] = xcorr(v, maxlags, 'coeff');
[corr_y, lags_y] = xcorr(y, maxlags, 'coeff');

%--------------------------------------------------------------------------
% Analysis plots
%--------------------------------------------------------------------------
% --- Bandpass filter
figure;
freqz(b_bp, a_bp);
subplot(211); 
ylim([-100 0]);
set(gca, 'YTick', -100:20:0);
title('Bandpass filter')

figure;
% --- Time domain signals
sp(1) = subplot(321);
plot(v);
ylabel('Amplitude');
xlabel('Sample index');
title('Filter input v(n)');
grid on;

sp(2) = subplot(322);
plot(y);
xlabel('Sample index');
title('Filter output y(n)');
grid on;

linkaxes(sp(1:2), 'x');

% --- Spectra
f      = linspace(0, 1, N/2);

sp(3) = subplot(323);
plot(f, 20*log10( abs(V_spec(1:N/2)))+ eps );
ylabel('Magnitude in dB');
xlabel('Normalized frequency (\times \pi rad/sample)');
grid on;

sp(4) = subplot(324);
plot(f, 20*log10(abs(Y_spec(1:N/2))) + eps );
xlabel('Normalized frequency (\times \pi rad/sample)');
grid on;

linkaxes(sp(3:4), 'x');

% --- Autocorrelations
sp(5) = subplot(325);
plot(lags_v, corr_v);
ylabel('Autocorrelation');
xlabel('Time lag \kappa');
set(gca, 'YLim', [-1 1]);
set(gca, 'XLim', [-maxlags maxlags]);
grid on;

sp(6) = subplot(326);
plot(lags_y, corr_y);
xlabel('Time lag \kappa');
set(gca, 'YLim', [-1 1]);
set(gca, 'XLim', [-maxlags maxlags]);
grid on;

linkaxes(sp(5:6), 'x');



