%--------------------------------------------------------------------------
% Signals and Systems II - Exercise 3
%
% An (unorthodox) example for frequency modulation. The input signal is an
% EEG signal of an epilepsy patient. The "content" of the signal is well
% below 100Hz and should be transformed to the audible frequency range
% (so-called auralization of EEG signals) by applying frequency modulation.%
%
% Nov. 2013
% Digital Signal Processing and System Theory
% Faculty of Engineering Christian-Albrechts-Universitt zu Kiel
% Jochen Withopf, jow@tf.uni-kiel.de
%--------------------------------------------------------------------------

clear all;
close all;
clc;

fileName = 'signals/eeg_005_F3.wav';
   

fm = 1000;   % Modulation frequency in Hz
k  = 500; % Modulation index

%--------------------------------------------------------------------------
% Load data and normalize to [-1 1]
%--------------------------------------------------------------------------
[data, fs] = wavread(fileName);
data       = data/max(abs(data));

%--------------------------------------------------------------------------
% Frequency modulation
%--------------------------------------------------------------------------

% --- Vector for time (2pi * t)
twoPiT   = 2*pi*linspace(0, length(data)/fs, length(data));
twoPiT   = twoPiT(:);

% --- Compute integral over the signal
twoPiInt = 2*pi*cumsum(data)/fs;

% --- The actucal modulation
data_mod = cos( twoPiT*fm + k*twoPiInt );

%--------------------------------------------------------------------------
% Plot: time-domain signal
%--------------------------------------------------------------------------
figure(1)
t = linspace(0, length(data)/fs, length(data));
plot(t, data);
grid on
xlim([t(1), t(end)])
ylabel('EEG signal');
xlabel('Time in seconds');

%--------------------------------------------------------------------------
% Plot: spectrogram
%--------------------------------------------------------------------------

% --- Compute spectrogram
[p_mod, f, t] = spectrogram(data_mod, 2048, 1024, 2048, fs);
[p_ecg, f, t] = spectrogram(data, 2048, 1024, 2048, fs);

figure(2);

subplot(2,1,1);
imagesc(t,f/1000,10*log10(abs(p_ecg))); 
axis xy;
ylim([0 10]);
title(['Original signal'] );
xlabel('Time in seconds'); 
ylabel('Frequency in kHz')
ylim([0 1]);

subplot(2,1,2);
imagesc(t,f/1000,10*log10(abs(p_mod))); 
axis xy;
ylim([0 10]);
title(['Frequency modulated signal, f_T=' int2str(fm) ...
       ' Hz, k=' int2str(k)] );
xlabel('Time in seconds'); 
ylabel('Frequency in kHz');


% --- Welch estimator for the signal PSD
% N = 50000;
% figure;
% pwelch(data, N, N/2, N, fs);
% xlim([0 0.3]);

%--------------------------------------------------------------------------
% Open in sound player
%--------------------------------------------------------------------------
%spclab(fs, data_mod)
