%--------------------------------------------------------------------------
% Signals and Systems II - Problem 5 (Aliasing effect)
%
% Select one of the signals (or load your own favourite audio file). It
% will be downsampled by the factor 'dn' with two methods to the signals:
%    - 'x_dn' by using an anti-aliasing lowpass first
%    - 'x_al' by simply discarding samples
%
% SPCLAB for comfortable file playback (and much more) is freely available
% at http://spclab.wikispaces.com/
%
% Jan. 2011
% Revised: Nov. 2011, Nov. 2012, Nov. 2013
% Digital Signal Processing and System Theory
% Faculty of Engineering Christian-Albrechts-Universitt zu Kiel
% Jochen Withopf, jow@tf.uni-kiel.de
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Set parameters and load audio file
%--------------------------------------------------------------------------
dn = 16;                                             % downsampling factor
filepath = 'signals\';

plot_orig = 0;

%--------------------------------------------------------------------------
% Load (or create) signals
%--------------------------------------------------------------------------
%[x fs] = wavread([filepath 'SeparateEntity.wav']);
[x fs] = wavread([filepath 'ce_jeu.wav']);
x = x(1:8.6*fs);

% --- or create as sinusoidal signal...
% fs = 44100;
% dur = 5;
% f0 = 2000;
% t = linspace(0,dur, dur*fs);
% x = cos(2*pi*f0*t);

%--------------------------------------------------------------------------
% If audio is stereo, mix both channel to one
%--------------------------------------------------------------------------
if size(x,2) == 2
    x = x(:,1) + x(:,2);
end

%--------------------------------------------------------------------------
% Compute downsampled signals:
%    - x_dn by using an anti-aliasing lowpass first
%    - x_al by simply discarding samples
%--------------------------------------------------------------------------
x_dn = resample(x, 1, dn);
x_al = x(1:dn:end);

%--------------------------------------------------------------------------
% Analysis of both signals
%--------------------------------------------------------------------------


% --- Plot spectrograms
colorLims = [-140 -20];   % for same value-to-color mapping in each plot
Nfft = 256;
ovl  = 240;

[S_dn, f, t, P_dn] = spectrogram(x_dn(1:4*fs/dn), hann(Nfft), ovl, Nfft, fs/dn, 'yaxis');
[S_al, f, t, P_al] = spectrogram(x_al(1:4*fs/dn), hann(Nfft), ovl, Nfft, fs/dn, 'yaxis');
[S,    fx, tx, P]  = spectrogram(x(1:4*fs), hann(Nfft*4), ovl*4, Nfft*4, fs, 'yaxis');

if plot_orig
    figure;
    surf(tx,fx/1000,10*log10(abs(P)),'EdgeColor','none');   
    axis xy; axis tight; colormap(jet); view(0,90);
    caxis(colorLims);
    xlabel('Time in seconds');
    ylabel('Frequency in kHz');

    % --- Open in spclab audioplayer
    spclab(round(fs), x);

else
    figure;
    subplot(221)
    surf(t,f,10*log10(abs(P_dn)),'EdgeColor','none');   
    axis xy; axis tight; colormap(jet); view(0,90);
    caxis(colorLims);
    %xlabel('Time in seconds');
    ylabel('Frequency in Hz');
    title('Downsampled');
    box on

    subplot(222)
    surf(t,f,10*log10(abs(P_al)),'EdgeColor','none');   
    axis xy; axis tight; colormap(jet); view(0,90);
    caxis(colorLims);
    %xlabel('Time in seconds');
    %ylabel('Frequency in Hz');
    title('Aliased');
    box on

    subplot(2,2,3:4)
    surf(t,f,10*log10(abs(P_al-P_dn)),'EdgeColor','none');   
    axis xy; axis tight; colormap(jet); view(0,90);
    caxis(colorLims);
    xlabel('Time in seconds');
    ylabel('Frequency in Hz');
    title('Difference');
    box on

    % --- Open in spclab audioplayer (first resample all audio to fs)
    x_dn_up = resample(x_dn, dn, 1);
    x_al_up = resample(x_al, dn, 1);
    N = min([length(x), length(x_dn_up), length(x_al_up)]);
    
    spclab(fs, x(1:N), x_dn_up(1:N), x_al_up(1:N));
end