%% Einfaches Beispiel fr die Quantisierung / Abtastung
% Datum: 26.11.2014
% Author: Jens Reermann
%
% Dieses Script simuliert sowohl die Quantisierung als auch die Abtastung
% und soll zum Verstndnis der auftretenden Effekte dienen
%
%% Reset
clear all;
close all;
clc;

%% Parameter
fact=2;   % Anzahl der Quantisierungsstufen zwischen -1 und 1: 2*fact+1

%% Bestimmung der Kennlinie
% Eingangssignal
analog_in = -2:0.001:2;

% Quantisierung
discrete_out=round(analog_in*fact)/fact;    % "Quantisierung"
discrete_out=max(-1,min(1,discrete_out));   % Begrenzung

% Plotten
figure(1);

subplot(2,1,1);
plot(analog_in,discrete_out,'r');
hold on;
plot(analog_in,analog_in,'b');
hold off;
xlabel('Analog');
ylabel('Digital');
legend([num2str(fact*2+1) ' Stufen'],'Ideal');

subplot(2,1,2);
plot(analog_in,analog_in-discrete_out);
xlabel('Analog');
ylabel('Fehler');

%% Sinus
% Eingangssignal
f_s=128;
f=1;
T=1;
n=0:1:T*f_s-1;
y = sin(2*pi*n*f/f_s);

% Quantisierung
y_d=round(y*fact)/fact; % "Quantisierung"
y_d=max(-1,min(1,y_d)); % Begrenzung

% Plotten
figure(2);

plot(n/f_s,y,'b');
hold on;
plot(n/f_s,y_d,'r-*');
hold off;
xlabel('Zeit in Sekunden');
ylabel('Amplitude');
legend('Analog','Digital');

%% Aliasing

% Signal (Summierung mehrerer sinusartiger Signale)
freqs=[2.5 4.1 40];     % Frequenzen
x=zeros(1, length(n));
for ii=1:length(freqs)
    x = x + sin(2*pi*n*freqs(ii)/f_s);
end

% Unterabtastung (Decimation): Simulation der Abtastung
dn=4;
f_s_dn=f_s/dn;
x_dn = resample(x, 1, dn);      % Mit Anti-Alias-Filter (stellt sicher, dass das Abtasttheorem erfllt ist)
x_al = x(1:dn:end);             % Ohne Filter

% FFT Berechnung
N_FFT=length(x);
X=fft(x);
N_FFT_dn=length(x_dn);
X_dn=fft(x_dn);
X_al=fft(x_al);

% Plotten
figure(3);
subplot(2,1,1);
plot(0:f_s/N_FFT:(N_FFT-1)*f_s/N_FFT,20*log10(abs(X)));
xlabel('Frequenz in Hz');
ylabel('Amplitudengang in dB');
xlim([0 f_s]);
title('Eingangssignal');

subplot(2,1,2);
plot(0:f_s_dn/N_FFT_dn:(N_FFT_dn-1)*f_s_dn/N_FFT_dn,20*log10(abs(X_dn)),'r');
hold on;
plot(0:f_s_dn/N_FFT_dn:(N_FFT_dn-1)*f_s_dn/N_FFT_dn,20*log10(abs(X_al)),'b');
xlabel('Frequenz in Hz');
ylabel('Amplitudengang in dB');
legend('Mit Anti-Aliasing-Filter','Ohne Anti-Aliasing-Filter');
title('Ausgangssignal');
xlim([0 f_s]);