%--------------------------------------------------------------------------
% Signals and Systems II - Problem 5 (Quantization noise)
%
% Example of a uniform quantizer with mid-rise charactersitic. Design the
% quantizer by setting v_max, v_min and w.
%
%
% SPCLAB for comfortable file playback (and much more) is freely available
% at http://spclab.wikispaces.com/
%
% Nov. 2010
% Revised: Nov. 2011, Nov. 2012, Nov. 2013
% Digital Signal Processing and System Theory
% Faculty of Engineering Christian-Albrechts-Universitt zu Kiel
% Jochen Withopf, jow@tf.uni-kiel.de
%--------------------------------------------------------------------------

close all
clear

% --- Control plots
plot_sig    = 1;
plot_hist   = 0;
plot_char   = 1;
plot_spclab = 1;

%--------------------------------------------------------------------------
% Define quantizer parameters
%--------------------------------------------------------------------------
v_max =  1;  % maximium amplitude
v_min = -1;  % minimum amplitude
w     =  4;  % number of bits (word length)

%--------------------------------------------------------------------------
% Load data
%--------------------------------------------------------------------------
data_type = 'speech'; % specify type of data: 'rand', 'sin', 'music' or 'speech'
fs  = 16000;          % sampling rate
dur = 2.5;            % duration in seconds

switch data_type
    case 'rand'
        data = rand(dur*fs, 1)-0.5;
        
    case 'sin'
        freq  = 500;                       % sinusoid's frequency in Hz
        dur   = 5/freq;                    % set length to 5 periods
        t_sin = linspace(0, dur, dur*fs);
        data  = sin(2*pi*freq*linspace(0, dur, dur*fs))';
        
    case 'music'
        [data, fs] = wavread('signals\ff_let_it_die.wav');
        %data = data(1:dur*fs);
        
    case 'speech'
        [data, fs] = wavread('signals\interview_heino.wav');
        %data = data(1:dur*fs);               
end

% --- Normalize amplitude to 1
data = data/max(abs(data)); 

%--------------------------------------------------------------------------
% Perform quantization
%--------------------------------------------------------------------------
delta      = (v_max - v_min)/2^w;               % delta is the same as Q in the lecture
codebook   = v_min+delta/2:delta:v_max-delta/2; % vector with reconstruction levels
partition  = v_min+delta:delta:v_max-delta;     % vector with decision boarders

% --- Quantization using the MATLAB-quantizer...
% [idx quant] = quantiz(data, partition, codebook); 

% --- ... or not (basically the same as quantiz)
idx = ones(length(data), 1);
for i=1:length(partition)
   idx = idx + double(data>partition(i)); 
end
quant = codebook(idx);
quant = quant(:); % make it a column vector
    
%--------------------------------------------------------------------------
% Make plots
%--------------------------------------------------------------------------
lw = 1;
t  = linspace(0, length(data)/fs, length(data));

% --- Plot original and quantized sinal
if plot_sig == 1
    figure;
    sp(1) = subplot(3,1,1:2);
    stairs(t, quant, 'Color', 'r', 'LineWidth', lw);
    hold on;
    plot(t, data, 'LineWidth', lw);
    axis([t(1), t(end), -1.1, 1.1])
    %xlabel('Time in seconds');
    ylabel('Amplitude');
    legend('Original singal', 'Quantized signal', 'Location', 'best');
    grid on

    sp(2) = subplot(3,1,3);
    plot(t, data-quant)
    xlim([t(1), t(end)])
    xlabel('Time in seconds');
    ylabel('Quantization error');
    grid on
    
    linkaxes(sp, 'x');
end

% --- Plot quantizer characteristic
if plot_char == 1
    figure; 
    v_in = [1.2*v_min, partition 1.2*v_max];
    v_q  = [codebook(1) ,codebook(2:end), codebook(end)];
    stairs(v_in, v_q);
    axis([v_in(1), v_in(end), v_min, v_max]);
    set(gca, 'XTick', v_in(2:end))
    set(gca, 'YTick', v_q(1:end-1))
    grid on;
    xlabel('Quantizer input signal v')
    ylabel('Quantizer output signal v_Q')
end

% --- Plot histograms of original and quantized signal
if plot_hist == 1
    numBins = 100;

    figure;
    subplot(211)
    hist(data, numBins);
    ylabel('Absolute frequency');
    title('Original signal');
    grid on

    subplot(212)
    hist(quant, numBins);
    title('Quantized signal');
    ylabel('Absolute frequency');
    xlabel('Sample amplitude');
    grid on
end

% --- Open spclab audioplayer
if plot_spclab == 1
    spclab(fs, data, quant, data-quant);
end